using System;
using System.Collections.Generic;
using System.Text;
using PernEdit.Plugin;
using PernEdit.WorldEntities;
using System.Drawing;
using PernEdit;

namespace PernEditor.Creators.Brushes
{
    /// <summary>
    /// flood fill brush
    /// </summary>
    public class BrushFillStamper : IBrushStamperPlugin
    {
        /// <summary>
        /// Stamps the current brush onto the specified layer, must use the layers
        /// stamp() function and not directly write to the the layer else you will
        /// bypass the undo stack.  Create a new layer with the data to stamp and them
        /// call the layer stamp function. (If the performance hit is a major concern
        /// view the undo stack and the layer.stamp() function and you can implement undo
        /// your self for that stamper)  (uses iodized stack based flood fill)
        /// </summary>
        /// <param name="brush">The brush.</param>
        /// <param name="layer">The layer.</param>
        /// <param name="area">The area to be affected by the stamp.</param>
        public void Stamp(TileBrush brush, Layer layer, Rectangle area)
        {
            int x = area.X;
            int y = area.Y;
 
            //the index of the tile we are trying to overwrite...
            //only want to fill tiles of this index
            short index = layer.Tiles[x + y * layer.Width].index;

            //snap to grid
            x = area.X / brush.BrushLayer.Width;
            y = area.Y / brush.BrushLayer.Height;

            Layer stamp = new Layer("stamper", layer.Width, layer.Height, layer.TileWidth, layer.TileHeight);
            
            Size brushSize = new Size(brush.BrushLayer.Width, brush.BrushLayer.Height);

            List<Point> stack = new List<Point>();

            stack.Add(new Point(x, y));

            while (stack.Count > 0)
            {
                Point p = stack[stack.Count - 1];
                stack.RemoveAt(stack.Count - 1);
                
                x = p.X;
                y = p.Y;

                int xoff = x * brushSize.Width;
                int yoff = y * brushSize.Height;         
                
                
                if (xoff < 0 || yoff < 0 || xoff >= layer.Width|| yoff >= layer.Height) 
                    continue;
                
                bool check = true;

                //make sure the tile we are looking at is the proper index
                for (int iy = yoff; iy < brushSize.Height + yoff && iy < layer.Height && check; iy++)
                    for (int ix = xoff; ix < brushSize.Width + xoff && ix < layer.Width; ix++)
                        if (layer.Tiles[ix + iy * layer.Width].index != index)
                        {
                            check = false;
                            break;
                        }
                
                //verify the tile is not exactly the same as the one we are painting
                if (check)
                {
                    check = false;

                    for (int iy = yoff; iy < brushSize.Height + yoff && iy < layer.Height && !check; iy++)
                        for (int ix = xoff; ix < brushSize.Width + xoff && ix < layer.Width; ix++)
                            if (stamp.Tiles[ix + iy * layer.Width].index != brush.BrushLayer.Tiles[(ix - xoff) + (iy - yoff) * brushSize.Width].index)
                            {
                                check = true;
                                break;
                            }
                }

                if (check)
                {
                    for (int iy = yoff; iy < brushSize.Height + yoff && iy < layer.Height; iy++)
                        for (int ix = xoff; ix < brushSize.Width + xoff && ix < layer.Width; ix++)
                            stamp.Tiles[ix + iy * layer.Width] = brush.BrushLayer.Tiles[(ix - xoff) + (iy - yoff) * brushSize.Width];

                    //add the next tiles to check
                    Point newP = new Point(x, y + 1);

                    if(!stack.Contains(newP))
                        stack.Add(newP);
                    
                    newP = new Point(x, y - 1);
                    
                    if (!stack.Contains(newP))
                        stack.Add(newP);
                    
                    newP = new Point(x + 1, y);
                    
                    if (!stack.Contains(newP))
                        stack.Add(newP);
                
                    newP = new Point(x - 1, y);
                    
                    if (!stack.Contains(newP))
                        stack.Add(newP);
                }
            }

            layer.Stamp(stamp, 0, 0, true);

        }

        /// <summary>
        /// Gets the bitmap.
        /// </summary>
        /// <param name="brush">The brush.</param>
        /// <returns></returns>
        public Bitmap GetBitmap(TileBrush brush, Rectangle area)
        {
            return brush.BrushLayer.GetBitmap();
        }

        /// <summary>
        /// Gets the icon to be used in the tool bar.
        /// </summary>
        /// <value>The icon.</value>
        public Bitmap Icon
        {
            get 
            {
                string path = Editor.GetPathFromRelative("Brushes\\FloodFillStamper\\fill.gif");
                return new Bitmap(path); 
            }
        }



        /// <summary>
        /// Enumerated type of the plugin
        /// </summary>
        /// <value></value>
        public PluginType Type
        {
            get { return PluginType.PLUGIN_TYPE_BRUSH; }
        }

        /// <summary>
        /// The name of the plugin.  Will be used by the app to list the plugin
        /// </summary>
        /// <value></value>
        public string Name
        {
            get { return "Flood Fill Brush"; }
        }

        /// <summary>
        /// Returns the name of the brush
        /// </summary>
        public override string ToString()
        {
            return Name;
        }
        /// <summary>
        /// GUID unique identifier for the plugin to avoid name clashes
        /// </summary>
        /// <value></value>
        public Guid ID
        {
            get { return new Guid("D9A7828D-6EFD-492a-8038-B3FB872347D6"); }
        }

        /// <summary>
        /// Version of the Pernedit Library the library was built against
        /// </summary>
        /// <value></value>
        public string Version
        {
            get { return "1.0.0.0"; }
        }

        /// <summary>
        /// Gets the tool tip to be displayed in the tool bar
        /// </summary>
        /// <value></value>
        public string MenuToolTip
        {
            get { return "Flood fills the space with the selected tile"; }
        }

     
    }
}
