using System;
using System.Collections.Generic;
using System.Text;
using PernEdit.WorldEntities;
using PernEdit.Plugin;
using System.Drawing;
using PernEdit;

namespace PernEditor.Creators.Brushes
{
    /// <summary>
    /// flood fills with tiles randomly selected from the current brush
    /// </summary>
    public class BrushRandomFill : IBrushStamperPlugin
    {
        private Random randomChooser = new Random((int)DateTime.Now.Ticks);

        /// <summary>
        /// Stamps the current brush onto the specified layer, must use the layers
        /// stamp() function and not directly write to the the layer else you will
        /// bypass the undo stack.  Create a new layer with the data to stamp and them
        /// call the layer stamp function. (If the performance hit is a major concern
        /// view the undo stack and the layer.stamp() function and you can implement undo
        /// your self for that stamper)
        /// </summary>
        /// <param name="brush">The brush.</param>
        /// <param name="layer">The layer.</param>
        /// <param name="area">The area to be affected by the stamp.</param>
         public void Stamp(TileBrush brush, Layer layer, Rectangle area)
         {
             int x = area.X;
             int y = area.Y;

            //the index of the tile we are trying to overwrite...
            //only want to fill tiles of this index
            short targetIndex = layer.Tiles[x + y * layer.Width].index;

            Layer stamp = new Layer("stamper", layer.Width, layer.Height, layer.TileWidth, layer.TileHeight);

            List<Point> stack = new List<Point>();

            stack.Add(new Point(x, y));

            while (stack.Count > 0)
            {
                Point p = stack[stack.Count - 1];
                stack.RemoveAt(stack.Count - 1);

                x = p.X;
                y = p.Y;

                if (x < 0 || y < 0 || x >= layer.Width || y >= layer.Height)
                    continue;

                if (stamp.Tiles[x + y * stamp.Width].index < 0 && layer.Tiles[x + y * stamp.Width].index == targetIndex)
                {

                    stamp.Tiles[x + y * layer.Width] = brush.BrushLayer.Tiles[randomChooser.Next(brush.BrushLayer.Tiles.Length)];

                    //add the next tiles to check
                    Point newP = new Point(x, y + 1);

                    if (!stack.Contains(newP))
                        stack.Add(newP);

                    newP = new Point(x, y - 1);

                    if (!stack.Contains(newP))
                        stack.Add(newP);

                    newP = new Point(x + 1, y);

                    if (!stack.Contains(newP))
                        stack.Add(newP);

                    newP = new Point(x - 1, y);

                    if (!stack.Contains(newP))
                        stack.Add(newP);
                }
            }

            layer.Stamp(stamp, 0, 0, true);
        }
        /// <summary>
        /// Gets the bitmap.
        /// </summary>
        /// <param name="brush">The brush.</param>
        /// <returns></returns>
        public Bitmap GetBitmap(TileBrush brush, Rectangle area)
        {
            return brush.BrushLayer.GetBitmap();
        }

        /// <summary>
        /// Gets the icon to be used in the tool bar.
        /// </summary>
        /// <value>The icon.</value>
        public Bitmap Icon
        {
            get { return new Bitmap(Editor.GetPathFromRelative("Brushes\\RandomFloodFillStamper\\fill_random.gif")); }
        }

        /// <summary>
        /// Enumerated type of the plugin
        /// </summary>
        /// <value></value>
        public PluginType Type
        {
            get { return PluginType.PLUGIN_TYPE_BRUSH; }
        }

        /// <summary>
        /// The name of the plugin.  Will be used by the app to list the plugin
        /// </summary>
        /// <value></value>
        public string Name
        {
            get { return "Brush Random Fill"; }
        }

        /// <summary>
        /// GUID unique identifier for the plugin to avoid name clashes
        /// </summary>
        /// <value></value>
        public Guid ID
        {
            get { return new Guid("D1BC471A-582C-442e-AF6B-F5702D028491"); }
        }

        /// <summary>
        /// Version of the Pernedit Library the library was built against
        /// </summary>
        /// <value></value>
        public string Version
        {
            get { return "1.0.0.0"; }
        }

        /// <summary>
        /// Returns the name of the brush
        /// </summary>
        public override string ToString()
        {
            return Name;
        }
        /// <summary>
        /// Gets the tool tip to be displayed in the tool bar
        /// </summary>
        /// <value></value>
        public string MenuToolTip
        {
            get {return "Flood fills the layer with randomly chosen tiles from the selected brush onto the layer"; }
        }
    }
}
